@@ -1,55 +1,73 @@
-(define-module (sdom ls)
-  #:use-module (srfi srfi-1)
-  #:use-module (srfi srfi-13)
-  #:use-module (sxml ssax)
-  #:use-module (sdom core)
-  #:export     (sdom:exception-code-parse-err
-		sdom:exception-code-serialize-err
+;; ls.scm: Load/Save exports and implementations for SDOM
+;; Copyright (C) 2010 Julian Graham
 
-		sdom:serialize
-		sdom:serialize-filter
-		sdom:xml->sdom))
+;; SDOM free software: you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
-(define sdom:exception-code-parse-err		81)
-(define sdom:exception-code-serialize-err	82)
+;; SDOM is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
 
-(sdom:register-feature! "LS" "3.0")
+;; You should have received a copy of the GNU General Public License
+;; along with SDOM.  If not, see <http://www.gnu.org/licenses/>.
 
-;; Add these options for the parser (which doesn't exist yet, I don't think...)
+#!r6rs
 
-(sdom:add-dom-config-parameter! "charset-overrides-xml-encoding" #t boolean?)
-(sdom:add-dom-config-parameter! "disallow-doctype" #f boolean?)
-(sdom:add-dom-config-parameter! "ignore-unknown-character-denormalizations"
-				#t boolean?)
-(sdom:add-dom-config-parameter! "namespaces" #t boolean?)
-(sdom:add-dom-config-parameter! "resource-resolver" '() procedure?)
-(sdom:add-dom-config-parameter! "supported-media-types-only" #f boolean?)
+(library (sdom ls (0 5))
+  (export sdom:exception-code-parse-err
+	  sdom:exception-code-serialize-err
+	  
+	  sdom:serialize
+	  sdom:serialize-filter
+	  sdom:xml->sdom)
+  (import (rnrs)
+	  (srfi :1)
+	  (srfi :13)
+	  (sxml ssax)
+	  (sdom core))
 
-;; Add these options for the serializer
+  (define sdom:exception-code-parse-err		81)
+  (define sdom:exception-code-serialize-err	82)
 
-(sdom:add-dom-config-parameter! "canonical-form" #f boolean?)
-(sdom:add-dom-config-parameter! "discard-default-content" #t boolean?)
-(sdom:add-dom-config-parameter! "format-pretty-print" #f boolean?)
-(sdom:add-dom-config-parameter! "xml-declaration" #t boolean?)
+  (sdom:register-feature! "LS" "3.0")
 
-(define serialize-error-internal
-  (lambda (node severity msg type excep data loc)
-    (let ((doc (sdom:get-dom-property node "sdom:owner-document")))
+  ;; Add these options for the parser (which doesn't exist yet, 
+  ;; I don't think...
+
+  (sdom:add-dom-config-parameter! "charset-overrides-xml-encoding" #t boolean?)
+  (sdom:add-dom-config-parameter! "disallow-doctype" #f boolean?)
+  (sdom:add-dom-config-parameter! "ignore-unknown-character-denormalizations"
+				  #t boolean?)
+  (sdom:add-dom-config-parameter! "namespaces" #t boolean?)
+  (sdom:add-dom-config-parameter! "resource-resolver" '() procedure?)
+  (sdom:add-dom-config-parameter! "supported-media-types-only" #f boolean?)
+  
+  ;; Add these options for the serializer
+  
+  (sdom:add-dom-config-parameter! "canonical-form" #f boolean?)
+  (sdom:add-dom-config-parameter! "discard-default-content" #t boolean?)
+  (sdom:add-dom-config-parameter! "format-pretty-print" #f boolean?)
+  (sdom:add-dom-config-parameter! "xml-declaration" #t boolean?)
+  
+  (define (serialize-error-internal node severity msg type excep data loc)
+    (let ((doc (sdom:document-element node)))
       (if (or (not (sdom:signal-error doc severity msg type excep data loc))
 	      (eqv? severity sdom:error-severity-fatal-error))
-	  (throw 'sdom:exception sdom:exception-code-serialize-err)))))
-
-(define s-elt 
-  (lambda (elt indent f fpp ddc xmld)
-    (let ((n (symbol->string (car elt)))
+	  (throw 'sdom:exception sdom:exception-code-serialize-err))))
+  
+  (define (s-elt elt indent f fpp ddc xmld)
+    (let ((n (sdom:tag-name elt))
 	  (spaces (if fpp (make-string indent #\space) "")))
       (string-append spaces
 		     "<" 
 		     n
-		     (let ((a (sdom:get-dom-property elt "sdom:attributes"))
+		     (let ((a (sdom:attributes elt))
 			   (g (lambda (x) (serialize x 1 f fpp ddc xmld))))
-		       (fold string-append "" (map g a)))
-		     (let ((c (sdom:get-dom-property elt "sdom:child-nodes")))
+		       (fold-right string-append "" (map g a)))
+		     (let ((c (sdom:child-nodes elt)))
 		       (if (not (null? c))
 			   (string-append ">" 
 					  (string #\nl)
@@ -57,100 +75,86 @@
 						     (serialize 
 						      x (+ 2 indent) f
 						      fpp ddc xmld))))
-					    (fold string-append "" (map g c)))
+					    (fold-right string-append 
+							"" (map g c)))
 					  spaces
 					  "</" 
 					  n 
 					  ">")
 			   "/>"))
-		     (if fpp (string #\nl) "")))))
-
-(define s-attr
-  (lambda (attr indent f fpp ddc xmld)
-    (let ((n (symbol->string (cadr attr))))
+		     (if fpp (string #\nl) ""))))
+  
+  (define (s-attr attr indent f fpp ddc xmld)
+    (let ((n (sdom:name attr)))
       (string-append (if fpp (make-string indent #\space) " ")
-		     n
-		     "=\""
-		     (sdom:get-dom-property attr "sdom:value")
-		     "\""))))
-
-(define serialize
-  (lambda (n indent f fpp ddc xmld)
-    (let ((type (sdom:node-type n)))
-      (cond ((eqv? type sdom:node-type-element) 
-	     (if (f n) (s-elt n indent f fpp ddc xmld) ""))
-	    ((eqv? type sdom:node-type-attr)
-	     (if (and (or (not ddc)
-			  (sdom:get-dom-property n "sdom:specified"))
-		      (f n))
-		 (s-attr n indent f fpp ddc xmld) ""))
-	    ((eqv? type sdom:node-type-text) 
-	     (if (f n) 
-		 (string-append (if fpp (make-string indent #\space) "")
-				(sdom:get-dom-property n "sdom:node-value")
-				(if fpp (string #\nl) ""))
-		 ""))
-	    ((eqv? type sdom:node-type-cdata-section)
-	     (if (f n) 
-		 (string-append (if fpp (make-string indent #\space) "")
-				"<![CDATA[" 
-				(sdom:get-dom-property n "sdom:node-value")
-				"]]>"
-				(if fpp (string #\nl) ""))
-		 ""))
-	    ((eqv? type sdom:node-type-entity-reference) 
-	     (if (f n) "" ""))
-	    ((eqv? type sdom:node-type-entity) 
-	     (if (f n) "" ""))
-	    ((eqv? type sdom:node-type-processing-instruction) 
-	     (if (f n) 
-		 (let ((target (sdom:get-dom-property n "sdom:target"))
-		       (data (sdom:get-dom-property n "sdom:data")))
-		   (if (string-contains target "?>") 
-		       (serialize-error-internal
-			n sdom:error-severity-fatal-error 
-			"illegal string in target" "target" '() target '()))
-		   (if (string-contains target "?>") 
-		       (serialize-error-internal
-			n sdom:error-severity-fatal-error 
-			"illegal string in data" "data" '() data '()))
-		   (string-append (make-string indent #\space)
-				  "<?" target " " data "?>"
-				  (string #\nl)) "")))
-	    ((eqv? type sdom:node-type-comment) 
-	     (if (f n) "" ""))
-	    ((eqv? type sdom:node-type-document)
-	     (if (f n)
-		 (s-elt (sdom:get-dom-property n "sdom:document-element") 
-			0
-			f 
-			fpp 
-			ddc
-			xmld)
-		 ""))
-	    ((eqv? type sdom:node-type-document-type) (if (f n) "" ""))
-	    ((eqv? type sdom:node-type-document-fragment) (if (f n) "" ""))
-	    ((eqv? type sdom:node-type-notation) (if (f n) "" ""))))))
-
-(define sdom:serialize-filter 
-  (lambda (node filter . port)
-    (let* ((doc (if (not (eqv? (sdom:node-type node) sdom:node-type-document))
-		    (sdom:get-dom-property node "sdom:owner-document")
-		    node))
+		     n "=\"" (sdom:value attr) "\"")))
+  
+  (define (serialize n indent f fpp ddc xmld)
+    (cond ((sdom:element? n) 
+	   (if (f n) (s-elt n indent f fpp ddc xmld) ""))
+	  ((sdom:attr? n)
+	   (if (and (or (not ddc) (sdom:specified n)) (f n))
+	       (s-attr n indent f fpp ddc xmld) ""))
+	  ((sdom:text-node? n) 
+	   (if (f n) 
+	       (string-append (if fpp (make-string indent #\space) "")
+			      (sdom:node-value n)
+			      (if fpp (string #\nl) ""))
+	       ""))
+	  ((sdom:cdata-section? n)
+	   (if (f n) 
+	       (string-append (if fpp (make-string indent #\space) "")
+			      "<![CDATA[" 
+			      (sdom:node-value n)
+			      "]]>"
+			      (if fpp (string #\nl) ""))
+	       ""))
+	  ((sdom:entity-reference? n) (if (f n) "" ""))
+	  ((sdom:entity? n) (if (f n) "" ""))
+	  ((sdom:processing-instruction? n) 
+	   (if (f n) 
+	       (let ((target (sdom:target n))
+		     (data (sdom:data n)))
+		 (if (string-contains target "?>") 
+		     (serialize-error-internal
+		      n sdom:error-severity-fatal-error 
+		      "illegal string in target" "target" '() target '()))
+		 (if (string-contains target "?>") 
+		     (serialize-error-internal
+		      n sdom:error-severity-fatal-error 
+		      "illegal string in data" "data" '() data '()))
+		 (string-append (make-string indent #\space)
+				"<?" target " " data "?>"
+				(string #\nl)) "")))
+	  ((sdom:comment? n) (if (f n) "" ""))
+	  ((sdom:document? n)
+	   (if (f n)
+	       (s-elt (sdom:document-element n)
+		      0
+		      f 
+		      fpp 
+		      ddc
+		      xmld)
+	       ""))
+	  ((sdom:document-type? n) (if (f n) "" ""))
+	  ((sdom:document-fragment? n) (if (f n) "" ""))
+	  ((sdom:notation? n) (if (f n) "" ""))))
+ 
+  (define (sdom:serialize-filter node filter . port)
+    (let* ((doc (if (sdom:document? node) node (sdom:owner-document node)))
 	   (r (serialize 
 	       node 0 filter
 	       (sdom:get-dom-config-parameter doc "format-pretty-print")
 	       (sdom:get-dom-config-parameter doc "discard-default-content")
 	       (sdom:get-dom-config-parameter doc "xml-declaration"))))
-      (if (null? port) r (display r (car port))))))
-
-(define sdom:serialize
-  (lambda (node . port)
+      (if (null? port) r (display r (car port)))))
+  
+  (define (sdom:serialize node . port)
     (let ((t (lambda (x) #t)))
       (if (null? port) 
 	  (sdom:serialize-filter node t)
-	  (sdom:serialize-filter node t (car port))))))
-
+	  (sdom:serialize-filter node t (car port)))))
+  
 ;; (define internal-dtd-parse-entity
 ;;   (lambda (port entities)
 ;;     ()))
@@ -276,3 +280,4 @@
 ;; 
 ;; 	     (result (reverse (parser port '()))))
 ;;      (cons '*TOP* result))))
+)
\ No newline at end of file
